"""
Measure traversal strategies

Stategy 1 avoids try/except at the possible cost of a second attribute access.
Stategy 2 uses try/except and propagates the initial AttributeError.

To run the performance tests type: python test_performance.py

$Id: test_performance.py,v 1.5 2005/10/30 17:33:28 stefan Exp $
"""

from unittest import TestCase, makeSuite
from unittest import TextTestRunner as TestRunner
from time import time

_marker = object()


def simpleTraverse_1(object, path_items, econtext):
    """Traverses a sequence of names, first trying attributes then items.
    """
    for name in path_items:
        next = getattr(object, name, _marker)
        if next is not _marker:
            object = next
        elif hasattr(object, '__getitem__'):
            object = object[name]
        else:
            object = getattr(object, name)
    return object


def simpleTraverse_2(object, path_items, econtext):
    """Traverses a sequence of names, first trying attributes then items.
    """
    for name in path_items:
        try:
            object = getattr(object, name)
        except AttributeError:
            if hasattr(object, '__getitem__'):
                object = object[name]
            else:
                raise
    return object


class Attr(object):
    next = None

def makeAttrChain(length):
    head = attr = Attr()
    for i in range(length):
        attr.next = Attr()
        attr = attr.next
    return head


class Item(dict):
    pass

def makeItemChain(length):
    head = item = Item()
    for i in range(length):
        item['next'] = Item()
        item = item['next']
    return head


class PerformanceTests(TestCase):
    # Tests are executed in alphabetical order

    def testAttrChainTraverse_1(self):
        print 'Attr traverse 1:',
        start = time()
        simpleTraverse_1(attr_chain, path_list, None)
        print '%.3fs' % (time()-start)

    def testAttrChainTraverse_2(self):
        print 'Attr traverse 2:',
        start = time()
        simpleTraverse_2(attr_chain, path_list, None)
        print '%.3fs' % (time()-start)

    def testItemChainTraverse_1(self):
        print 'Item traverse 1:',
        start = time()
        simpleTraverse_1(item_chain, path_list, None)
        print '%.3fs' % (time()-start)

    def testItemChainTraverse_2(self):
        print 'Item traverse 2:',
        start = time()
        simpleTraverse_2(item_chain, path_list, None)
        print '%.3fs' % (time()-start)


if __name__ == '__main__':

    num = 50000
    attr_chain = makeAttrChain(num)
    item_chain = makeItemChain(num)
    path_list = ['next']*num

    suite = makeSuite(PerformanceTests)
    TestRunner(verbosity=0).run(suite)

